@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+from __future__ import print_function
 
 # This connects to the system Tor (by default on control port 9151)
 # and adds a new hidden service configuration to it.
@@ -34,28 +34,30 @@ def main(reactor):
     )
 
     def on_progress(percent, tag, msg):
-        print('%03d: %s' % (percent, msg))
+        print('{:3.0f}%: {}'.format(percent, msg))
     txtorcon.IProgressProvider(ep).add_progress_listener(on_progress)
 
-    print "Starting site"
+    print("Starting site")
     port = yield ep.listen(server.Site(Simple()))
     host = port.getHost()
 
-    print "Site started. Available at http://{}:{}".format(host.onion_uri, host.onion_port)
-    print "Private key:\n{}".format(host.onion_key)
+    print("Site started. Available at http://{}:{}".format(host.onion_uri, host.onion_port))
+    print("Private key:\n{}".format(host.onion_key))
 
-    # XXX how to get the HiddenService instance? or the
-    # IOnionBlahFoo-implementing thing, I mean
-    print("port", dir(port))
-    print("port", type(port))
-    print("port", port.__provides__)
-    print("host", dir(host))
+    # if you have need of the actual IOnionService object (e.g. an
+    # EphemeralHiddenService or FilesystemHiddenService instance) then
+    # you can get it via the port (which itself is a
+    # TorOnionListeningPort instance)
+    service = port.onion_service
 
-    print("ports", host.public_ports)
+    print("Ports:", service.ports)
+    print("Hostname:", service.hostname)
+    print("Private key:", service.private_key)
 
     # wait forever; obviously you could do other work here
     d = defer.Deferred()
     yield d
 
 
-react(main)
+if __name__ == '__main__':
+    react(main)
