@@ -6,7 +6,7 @@ from __future__ import unicode_literals
 from __future__ import with_statement
 
 import time
-import datetime
+from datetime import datetime
 
 from twisted.python.failure import Failure
 from twisted.python import log
@@ -39,16 +39,12 @@ class Circuit(object):
 
     :ivar state:
         contains a string from Tor describing the current state of the
-        stream. From control-spec.txt section 4.1.2, these are:
-            - NEW: New request to connect
-            - NEWRESOLVE: New request to resolve an address
-            - REMAP: Address re-mapped to another
-            - SENTCONNECT: Sent a connect cell along a circuit
-            - SENTRESOLVE: Sent a resolve cell along a circuit
-            - SUCCEEDED: Received a reply; stream established
-            - FAILED: Stream failed and not retriable
-            - CLOSED: Stream closed
-            - DETACHED: Detached from circuit; still retriable
+        stream. From control-spec.txt section 4.1.1, these are:
+           - LAUNCHED: circuit ID assigned to new circuit
+           - BUILT: all hops finished, can now accept streams
+           - EXTENDED: one more hop has been completed
+           - FAILED: circuit closed (was not built)
+           - CLOSED: circuit closed (was built)
 
     :ivar purpose:
         The reason this circuit was built. Values can currently be one
@@ -123,7 +119,7 @@ class Circuit(object):
             # strip off milliseconds
             t = self.flags['TIME_CREATED'].split('.')[0]
             tstruct = time.strptime(t, TIME_FORMAT)
-            self._time_created = datetime.datetime(*tstruct[:7])
+            self._time_created = datetime(*tstruct[:7])
         return self._time_created
 
     def listen(self, listener):
@@ -157,7 +153,7 @@ class Circuit(object):
         d.addCallback(close_command_is_queued)
         return self._closing_deferred
 
-    def age(self, now=datetime.datetime.utcnow()):
+    def age(self, now=None):
         """
         Returns an integer which is the difference in seconds from
         'now' to when this circuit was created.
@@ -166,6 +162,8 @@ class Circuit(object):
         """
         if not self.time_created:
             return None
+        if now is None:
+            now = datetime.utcnow()
         return (now - self.time_created).seconds
 
     def _create_flags(self, kw):
