@@ -17,6 +17,7 @@ from twisted.python import log
 from twisted.internet import defer, error, protocol
 from twisted.internet.interfaces import IReactorTime
 from twisted.internet.endpoints import TCP4ClientEndpoint
+from twisted.internet.endpoints import UNIXClientEndpoint
 
 from txtorcon.torcontrolprotocol import parse_keywords, TorProtocolFactory, DEFAULT_VALUE
 from txtorcon.util import delete_file_or_tree, find_keywords, find_tor_binary
@@ -115,6 +116,7 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         self._setup_complete = False
         self._did_timeout = False
         self._timeout_delayed_call = None
+        self._on_exit = []  # Deferred's we owe a call/errback to when we exit
         if timeout:
             if not ireactortime:
                 raise RuntimeError(
@@ -124,6 +126,30 @@ class TorProcessProtocol(protocol.ProcessProtocol):
             self._timeout_delayed_call = ireactortime.callLater(
                 timeout, self.timeout_expired)
 
+    def quit(self):
+        """
+        This will terminate (with SIGTERM) the underlying Tor process.
+
+        :returns: a Deferred that callback()'s (with None) when the
+            process has actually exited.
+        """
+
+        try:
+            self.transport.signalProcess('TERM')
+            d = defer.Deferred()
+            self._on_exit.append(d)
+
+        except error.ProcessExitedAlready:
+            self.transport.loseConnection()
+            d = defer.succeed(None)
+        return d
+
+    def _signal_on_exit(self, reason):
+        to_notify = self._on_exit
+        self._on_exit = []
+        for d in to_notify:
+            d.callback(None)
+
     def outReceived(self, data):
         """
         :api:`twisted.internet.protocol.ProcessProtocol <ProcessProtocol>` API
@@ -179,6 +205,9 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         all([delete_file_or_tree(f) for f in self.to_delete])
         self.to_delete = []
 
+    def processExited(self, reason):
+        self._signal_on_exit(reason)
+
     def processEnded(self, status):
         """
         :api:`twisted.internet.protocol.ProcessProtocol <ProcessProtocol>` API
@@ -201,6 +230,7 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         if self.connected_cb:
             self.connected_cb.errback(err)
             self.connected_cb = None
+        self._signal_on_exit(status)
 
     def progress(self, percent, tag, summary):
         """
@@ -413,10 +443,16 @@ def launch_tor(config, reactor,
         config.CookieAuthentication = 1
         config.__OwningControllerProcess = os.getpid()
         if connection_creator is None:
-            connection_creator = functools.partial(
-                TCP4ClientEndpoint(reactor, 'localhost', control_port).connect,
-                TorProtocolFactory()
-            )
+            if str(control_port).startswith('unix:'):
+                connection_creator = functools.partial(
+                    UNIXClientEndpoint(reactor, control_port[5:]).connect,
+                    TorProtocolFactory()
+                )
+            else:
+                connection_creator = functools.partial(
+                    TCP4ClientEndpoint(reactor, 'localhost', control_port).connect,
+                    TorProtocolFactory()
+                )
     else:
         connection_creator = None
 
@@ -1101,7 +1137,7 @@ class TorConfig(object):
 
         if control is None:
             self._protocol = None
-            self.__dict__['_slutty_'] = None
+            self.__dict__['_accept_all_'] = None
 
         else:
             self._protocol = ITorControlProtocol(control)
@@ -1159,7 +1195,7 @@ class TorConfig(object):
         self.__dict__['_protocol'] = proto
 
         # FIXME some of this is duplicated from ctor
-        del self.__dict__['_slutty_']
+        del self.__dict__['_accept_all_']
         self.__dict__['post_bootstrap'] = defer.Deferred()
         if proto.post_bootstrap:
             proto.post_bootstrap.addCallback(self.bootstrap)
@@ -1181,13 +1217,20 @@ class TorConfig(object):
         attributes we need in the constructor without uusing __dict__
         all over the place.
         """
-        has_setup_attr = lambda o: '_setup_' in o.__dict__
-        has_slutty_attr = lambda o: '_slutty_' in o.__dict__
-        is_hidden_services = lambda s: s.lower() == "hiddenservices"
+
+        # appease flake8's hatred of lambda :/
+        def has_setup_attr(o):
+            return '_setup_' in o.__dict__
+
+        def has_accept_all_attr(o):
+            return '_accept_all_' in o.__dict__
+
+        def is_hidden_services(s):
+            return s.lower() == "hiddenservices"
 
         if has_setup_attr(self):
             name = self._find_real_name(name)
-            if not has_slutty_attr(self) and not is_hidden_services(name):
+            if not has_accept_all_attr(self) and not is_hidden_services(name):
                 value = self.parsers[name].validate(value, self, name)
             if isinstance(value, list):
                 value = _ListWrapper(
@@ -1212,13 +1255,13 @@ class TorConfig(object):
         to be called''
         """
         rn = self._find_real_name(name)
-        if '_slutty_' in self.__dict__ and rn in self.unsaved:
+        if '_accept_all_' in self.__dict__ and rn in self.unsaved:
             return self.unsaved[rn]
         self._maybe_create_listwrapper(rn)
         return self.config[rn]
 
     def __contains__(self, item):
-        if item in self.unsaved and '_slutty_' in self.__dict__:
+        if item in self.unsaved and '_accept_all_' in self.__dict__:
             return True
         return item in self.config
 
