@@ -11,7 +11,9 @@ import hashlib
 import shutil
 import socket
 import subprocess
+import ipaddress
 import struct
+import re
 
 from twisted.internet import defer
 from twisted.internet.interfaces import IProtocolFactory
@@ -61,16 +63,10 @@ def maybe_create_db(path):
     except IOError:
         return None
 
-city, asn, country = list(map(maybe_create_db,
-                         ("/usr/share/GeoIP/GeoLiteCity.dat",
-                          "/usr/share/GeoIP/GeoIPASNum.dat",
-                          "/usr/share/GeoIP/GeoIP.dat")))
 
-try:
-    import ipaddr as _ipaddr
-    ipaddr = _ipaddr
-except ImportError:
-    ipaddr = None
+city = maybe_create_db("/usr/share/GeoIP/GeoLiteCity.dat")
+asn = maybe_create_db("/usr/share/GeoIP/GeoIPASNum.dat")
+country = maybe_create_db("/usr/share/GeoIP/GeoIP.dat")
 
 
 def is_executable(path):
@@ -127,10 +123,9 @@ def maybe_ip_addr(addr):
     TODO consider explicitly checking for .exit or .onion at the end?
     """
 
-    if ipaddr is not None:
-        try:
-            return ipaddr.IPAddress(addr)
-        except ValueError:
+    try:
+        return ipaddress.ip_address(addr)
+    except ValueError:
             pass
     return str(addr)
 
@@ -145,6 +140,8 @@ def find_keywords(args, key_filter=lambda x: not x.startswith("$")):
     with "$hash=name" looks like a keyword argument (but it isn't). If you
     don't want this, override the "key_filter" argument to this method.
 
+    :param args: a list of strings, each with one key=value pair
+
     :return:
         a dict of key->value (both strings) of all name=value type
         keywords found in args.
@@ -290,3 +287,32 @@ def available_tcp_port(reactor):
     address = port.getHost()
     yield port.stopListening()
     defer.returnValue(address.port)
+
+
+def unescape_quoted_string(string):
+    r'''
+    This function implementes the recommended functionality described in the
+    tor control-spec to be compatible with older tor versions:
+
+      * Read \\n \\t \\r and \\0 ... \\377 as C escapes.
+      * Treat a backslash followed by any other character as that character.
+
+    Except the legacy support for the escape sequences above this function
+    implements parsing of QuotedString using qcontent from
+
+    QuotedString = DQUOTE *qcontent DQUOTE
+
+    :param string: The escaped quoted string.
+    :returns: The unescaped string.
+    :raises ValueError: If the string is in a invalid form
+                        (e.g. a single backslash)
+    '''
+    match = re.match(r'''^"((?:[^"\\]|\\.)*)"$''', string)
+    if not match:
+        raise ValueError("Invalid quoted string", string)
+    string = match.group(1)
+    # remove backslash before all characters which should not be
+    # handeled as escape codes by string.decode('string-escape').
+    # This is needed so e.g. '\x00' is not unescaped as '\0'
+    string = re.sub(r'((?:^|[^\\])(?:\\\\)*)\\([^ntr0-7\\])', r'\1\2', string)
+    return string.decode('string-escape')
